//<t>   Definitions for Disk, Directory, and File Functions

//<c>   Copyright 1980-1996 by Gerry J. Danen; all rights reserved.


#ifndef _DEFD_GD_FILES_H_
#define _DEFD_GD_FILES_H_ 1


#include<fcntl.h>
#include<io.h>
#include<stat.h>
#include"ms_struc.h"


#define         READ_ONLY       0x01    //  Bits used in attribute
#define         HIDDEN          0x02    //  field of directory entry
#define         SYSTEM          0x04
#define         VOL_ENTRY       0x08
#define         DIRECTORY       0x10
#define         ARCHIVE         0x20
#define         FSPEC           struct file_spec
#define         DIRLIST         struct directory_list
#define         SUBDIR          struct subdir_list


                                        //  File information structure
FSPEC                                   //  used by FindFirstFile and
{                                       //  FindNextFile
    UCHAR       fname[13]       ;       //  File name xxxxxxxx.xxx
    UINT32      fsize           ;       //  File size in bytes
    UINT32      fdate_st        ;       //  Date stamp yyyymmdd
    UINT32      ftime_st        ;       //  Time stamp hhmmss00
    UCHAR       fdate_tx[10]    ;       //  Date stamp dd-mmm-yy
    UCHAR       ftime_tx[ 9]    ;       //  Time stamp hh:mmx or hh:mm:ss
    INT16       fattr           ;       //  Attribute  (1 byte)
    INT16       f_at_read_only  ;       //  Attribute bit read-only
    INT16       f_at_hidden     ;       //  Attribute bit hidden
    INT16       f_at_system     ;       //  Attribute bit system
    INT16       f_at_archive    ;       //  Attribute bit archive
    INT16       f_at_vol_label  ;       //  Attribute bit volume label
    INT16       f_at_sub_dir    ;       //  Attribute bit sub-directory
    INT16       f_at_reserved_1 ;       //  Attribute bit reserved
    INT16       f_at_reserved_2 ;       //  Attribute bit reserved
};


DIRLIST
{
    UCHAR       name [ 8+1]     ;       //  name of file
    UCHAR       ext  [ 3+1]     ;       //  extension
    UCHAR       date [11+1]     ;       //  dd-mmm-yy or dd-mmm-yyyy
    UCHAR       time [ 8+1]     ;       //  hh:mmXM or hh:mm:ss
    UCHAR       size [13+1]     ;       //  9,999,999,999
    UCHAR       s_date[ 4]      ;       //  date for sorting, no \0
    UCHAR       s_time[ 3]      ;       //  time for sorting, no \0
    UCHAR       s_size[ 5]      ;       //  size for sorting, no \0
    FLAG        at_read_only    ;       //  attr: read-only
    FLAG        at_archive      ;       //  attr: archive
    FLAG        at_hidden       ;       //  attr: hidden
    FLAG        at_system       ;       //  attr: system
    FLAG        at_sub_dir      ;       //  attr: sub-directory
    FLAG        marked          ;       //  entry marked YES/NO
};


SUBDIR
{
    UCHAR       path[FS_LEN]    ;       //  path name
    INT16       cur             ;       //  current entry number
    INT16       entries         ;       //  valid # of entries in array
    UINT32      total_bytes     ;       //  total bytes in all files
};


//      Prototypes
// // INT16   AbsRead(INT16 drive,UINT16 nsect,UINT16 sector,UCHAR *buffer);
// void    bload_file(STRING fname,INT16 cls_fg,INT16 cls_bg);
// void    bsave_file(STRING fname);
// INT16   compare_files(UCHAR file1[],UCHAR file2[]);
// INT16   copy_file(UCHAR oldfile[],UCHAR newfile[],INT16 verify,INT16 confirm_overwrite,INT16 delete_after);
// void    CouldNotOpen(STRING fname,STRING mode);
// INT16   CurrentDrive(void);
// INT16   CurrentPath(STRING pdir);
// INT16   CurrPathDrive(INT16 drive,STRING pdir);
// UINT16  Date2Fstamp(INT16 year,INT16 month,INT16 day);
// UINT32  DiskSpace(INT16 *drive,UINT16 *p_av_clus,UINT32 *p_to_clus,UINT16 *p_sect_size,UINT16 *p_clus_size,UINT32 *p_tbytes);
// INT16   dr_expand_path(UCHAR *inpath,UCHAR *drive,UCHAR *dir,UCHAR *fname,UCHAR *ext);
// BOOL    dr_is_cdrom(UCHAR drv_number);
// BOOL    dr_is_directory(STRING pathname);
// BOOL    dr_is_networked(UCHAR drv_number);
// BOOL    dr_is_remote(UCHAR drive_letter);
// BOOL    dr_is_removable(UCHAR drv_number);
// BOOL    dr_is_subst(UCHAR drv,STRING real_path);
// BOOL    dr_mkdir(STRING dirname,BOOL go_there);
// STRING  dr_q_fname(STRING infile);
// STRING  dr_remove_trailer_from_path(UCHAR path[]);
// int     dr_type(UCHAR c);
// INT16   dr_valid_path(UCHAR fullspec[],UCHAR path_out[],UCHAR rest_out[]);
// int     ds_avail_drives(UCHAR drives[]);
// ULONG   ds_clustersize(UCHAR drive);
// int     ds_create_label(UCHAR drive,UCHAR label[]);
// BOOL    ds_get_media_info(UCHAR drive,STRING label,STRING serial,STRING fsystem);
// BOOL    ds_get_serial_number(UCHAR drive,UCHAR serial[]);
// BOOL    ds_ready(UCHAR drive);
// int     ds_remove_label(UCHAR drive);
// BOOL    ds_set_media_info(UCHAR drive,STRING label,STRING serial,STRING fsystem);
// BOOL    ds_set_serial_number(UCHAR drive,UCHAR serial[]);
// BOOL    ds_set_volume_label(UCHAR drive,STRING label);
// BOOL    ds_vol_label(STRING given_drive,STRING vl_drive,UCHAR vl_label[],INT32 *vl_date,INT32 *vl_time);
// INT16   ExtractFileExt(STRING dest,STRING in_str,BOOL make_upper);
// UINT16  FATvalue(INT16 is12,UINT16 cluster,UCHAR fat[]);
// INT16   FindFileInPath(STRING FoundWhere,STRING FileName);
// INT16   FindFirstFile(UCHAR *pname,INT16 sattr,FSPEC *pfile);
// INT16   FindNextFile(FSPEC *pfile);
// INT16   flretdta(ADS *pdta_ads);
// INT16   flsetdta(ADS *pdta_ads);
// int     fl_create_extfcb(e_fcb_t *fcb);
// INT16   fl_delete(STRING f_name);
// int     fl_delete_extfcb(e_fcb_t *fcb);
// INT16   fl_read1(int fh,UCHAR dest[],INT16 maxchar,INT16 *endfile);
// INT16   fl_rename(STRING old_file,STRING new_file);
// INT16   fl_split_path(STRING PathName,STRING FileSpec);
// STRING  fl_translate_name(STRING name);
// INT16   Fstamp2date(UINT16 fdate,INT16 *pyear,INT16 *pmonth,INT16 *pday);
// INT16   Fstamp2time(UINT16 ftime,INT16 *phour,INT16 *pminute,INT16 *psecond);
// INT16   GetAttrib(UCHAR *fpath,INT16 *at_ro,INT16 *at_hidden,INT16 *at_system,INT16 *at_vol,INT16 *at_dir,INT16 *at_arch);
// INT16   GetClusterSize(INT16 drive);
// struct disk_table FAR * GetDiskTablePtr(INT16 drv);
// UCHAR   GetDrive(void);
// INT16   GetFileInfo(FILE *fptr,UINT32 *pfdate,UINT32 *pftime);
// INT16   GetFreeDiskSpace(INT16 drive,UINT16 *avail,UINT16 *total,UINT16 *sectsize);
// BOOL    list_file(STRING fname);
// void    list_files(STRING fspec);
// void    log_wr(INT16 msg_type,INT16 msg_err,INT16 dump_scr,STRING msg);
// INT16   NormalizePath(UCHAR *pfile,UCHAR *pnorm,INT16 *plast);
// INT16   ParseFileName(STRING filename,e_fcb_t *fcb,INT16 mode);
// INT16   ReadBinFromFile(FILE *fdin,INT16 howmany,UCHAR *buffer,INT16 *endfile);
// INT16   ReadLine(FILE *fhand,UCHAR *dest,INT16 maxchar,INT16 flush,INT16 *endfile);
// int     SearchFirstFile(e_fcb_t *fcb);
// int     SearchNextFile(e_fcb_t *fcb);
// INT16   SelectDrive(INT16 drive);
// void    serial(UCHAR drv);
// INT16   SetDrive(UCHAR letter);
// INT16   SetDTA(UCHAR *buffer);
// INT16   SetFileInfo(FILE *fptr,UINT32 fdate,UINT32 ftime);
// void    set_fstamp_format(INT16 format);
// UINT16  Time2Fstamp(INT16 hour,INT16 minute,INT16 second);
// int     truename(char *filename);
// int     unpack_archive(STRING arc,STRING where,UINT32 file_date,UINT32 file_time);
// BOOL    ValidDrive(int drive);
// INT16   ValidPath(UCHAR *fullspec,UCHAR *path_out,UCHAR *rest_out);
// BOOL    valid_drive(UCHAR drive);
// INT16   WriteBinToFile(FILE *fd_out,INT16 howmany,UCHAR *buffer);


void    DIR_exit(DIRLIST *dir_e[]);
void    DIR_file_spec(STRING spec);
INT16   DIR_find(SUBDIR *dir_h,DIRLIST *dir_e[],UCHAR *fspec,INT16 attribs);
INT16   DIR_format_entry(SUBDIR *dir_h,DIRLIST *dir_e[],INT16 this_one,STRING pr_line,FLAG f_screen);
INT16   DIR_init(INT16 dir_max,SUBDIR *dir_h,DIRLIST *dir_e[]);
void    DIR_ret_filespec(UCHAR *fspec);
void    DIR_set_filespec(UCHAR *fspec);
void    DIR_show(SUBDIR *dir_h,DIRLIST *dir_e[],UCHAR *mark_msg,INT16 ercode,INT16 mark_em);
void    DIR_sort(SUBDIR *dir_h,DIRLIST *dir_e[],INT16 sort_type);
void    DIR_sort_spec(void);
INT16   FindFirstFile(UCHAR *pname,INT16 sattr,FSPEC *pfile);
INT16   FindNextFile(FSPEC *pfile);


#endif  // avoid duplicate definition
