/*
<t>     Scan All Directories




        Convert to use _dos_findfirst(), _dos_findnext()




        SCANDIR scans the disk for the specified file(s), displays the
        information and gives the user the opportunity to alter the directory.
        SCANDIR is invoked by the command

                        scandir /F filename /D directory /L level /T /N

                                OR

                        scandir filename directory level

        The parameters have the following meaning:

                /F      filename for which the search is made.  Wildcards may be
                        used, in which case any match is displayed.  The default
                        is to display all files, i.e., *.*

                /D      directory where the search is to begin.  The default is
                        to start the search at the current directory.

                /L      The number of levels of subdirectories to search.  The
                        default is to seach all subdirectories.

                /T      Terse mode.  If the switch is not specified, full
                        displays are presented.

                /N      Check for trailing NULLs in the file.  NULLs are
                        typically inserted by some programs as padding.

        For each match complete directory information is presented.  With this
        version, the user may delete the file, continue, quit, or quit and stay
        in the same directory.  The appropriate resonses when the file
        information is displayed are:

                E       Erase the file
                Q       Quit, but return to the home directory
                F       Quit, but stay in the directory begin processed
*/
#include"gd_tools.h"
#include"gd_files.h"
#include"gd_ansi.h"
#include<fcntl.h>
#include<io.h>


#define pgVERS_ "V3.20"
#define pgTITL_ "Scan Directories for Files"
#define pgCOPR_ "1987"


void    scan_dir(STRING pdir,INT16 level);


UCHAR   filechk[9], extchk[4] ;                 // File pattern to look for
UCHAR   home_dir[MAX_PATH_LEN] ;                // Home directory
UCHAR   tome_dir[MAX_PATH_LEN] ;                // Home dir. on target drv
INT16   home_drv ;                              // Home drive
INT16   verbose = TRUE ;
INT16   N_switch = FALSE ;


void    do_quit( int ecode, STRING emsg )
{
    if ( strlen(emsg) )
        printf( "\n%s\n", emsg );
    exit( ecode );
}


void    do_help( void )
{
    printf( "\n%s /F filename /D directory /L level /T /N\n\n"
            "\t/F = file name(s) to look for\n"
            "\t/D = starting [drive:]directory\n"
            "\t/L = how many levels of sub-directories to search (0=none)\n"
            "\t/T = terse mode, doesn't wait for reply\n"
            "\t/N = check for NULL padding in files", get_global_program_name() );
    do_quit( 1, " " );
}


void    fix_str( STRING s )
{
    _strupr( s );
    st_trim( s );
}


INT16   matched( STRING pfile )
/*
  Name          matched -- Test for file name match

  Synopsis      check = matched( pfile );
                INT16 check     If the file name matches,
                                TRUE is returned, otherwise FALSE.
                STRING pfile    The file name to check

  Description   The file name is checked against the pattern in the global
                variables filechk and extchk.  For the comparison to return a
                value of TRUE, each character must either match the
                corresponding character in filechk (or extchk), or the filechk
                character must be a wildcard (?).  The file name is copied to
                strings of normal form so that the comparison is done on strings
                of equal length.  A normal form has all characters set to null
                (0), and then the first characters replaced by the actual
                characters from the string.

  Returns       check           TRUE if the file name matches; FALSE otherwise.
*/
{
    UCHAR       filematch[9], extmatch[4], ch ;
    INT16       i, do_ext = TRUE, match = TRUE ;

    if ( StrFindChar( '.', pfile ) < 0 )
        do_ext = FALSE ;

    //  First initialize the strings to check for a match.

    StrFill( filematch, EOS, 9 );
    StrFill( extmatch , EOS, 4 );
    for ( i = 0 ; ch = *pfile++ ; i++ )
    {
        if ( ch == '.' )
            break ;
        else
            filematch[i] = ch ;
    }
    if ( do_ext )
    {
        for ( i = 0 ; extmatch[i] = *pfile++ ; i++ )
            ;
    }

    //  Now check to see if a match exists.

    LOOP( i, 0, 7, 1 )
    {
        switch ( filechk[i] )
        {
        case '?':
            break ;
        case UCHARMAX :
            if ( filematch[i] != EOS )
                goto no_match ;
            break ;
        default:
            if ( filechk[i] != filematch[i] )
                goto no_match ;
        }
    }
    LOOP( i, 0, 2, 1 )
    {
        switch ( extchk[i] )
        {
        case '?':
            break ;
        case UCHARMAX :
            if ( extmatch[i] != EOS )
                goto no_match ;
            break ;
        default:
            if ( extchk[i] != extmatch[i] )
                goto no_match ;
            break ;
        }
    }
    goto matched_done ;

no_match:
    match = FALSE ;

matched_done:
    return( match );
}


void    save_DTA( UCHAR dest[] )
{
    extern      UCHAR   FAR_MEM ff_temp_DTA_[] ;
    INT16       i ;

    LOOP( i, 0, 47, 1 )
    {
        dest[ i ] = ff_temp_DTA_[ i ] ;
    }
}


void    rest_DTA( UCHAR from[] )
{
    extern      UCHAR   FAR_MEM ff_temp_DTA_[] ;
    INT16       i ;

    LOOP( i, 0, 47, 1 )
    {
        ff_temp_DTA_[ i ] = from[ i ] ;
    }
}


void    go_home( void )
{
    _chdir( tome_dir );
    SelectDrive( home_drv );                    // return "home"
    _chdir( home_dir );
}


INT16   check_for_nulls( STRING path, STRING fname )
{
    UCHAR       buf[2], fspec[ FS_LEN ];
    INT16       fh, ret=FALSE ;

    sprintf( fspec, "%s\\%s", path, fname );

    fh = _open( fspec, (int) (O_RDONLY | O_BINARY) );
    if ( fh < 0 )
        return( ERROR );                // open failure
    _lseek( fh, -1L, SEEK_END );        // find last byte in file
    _read( fh, buf, 1 );
    if ( buf[0] == (UCHAR) 0 )
        ret = TRUE ;                    // NULL found at eof
    _close( fh );
    return( ret );
}


void    scan_dir( STRING pdir, INT16 level )
/*
  Name          scan_dir -- Scan a directory for a file name

  Synopsis      scan_dir( pdir, level );
                STRING pdir     The directory to scan
                INT16 level     The subdirectory level.  If zero no
                                subdirectory should be scanned.

  Description   scan_dir is a recursive routine which scans the specified
                directory for all occurrences of the file name.  If verbose mode
                is set (again, a global variable) a file information is
                displayed, and changes are requested from the user.  If a
                subdirectory is encountered, and level is positive, scan_dir
                calls itself to process the subdirectory.  The function matched
                is used to check that the next file in the directory is match
                with that specified on the command line when SCANDIR is invoked.
*/
{
    FSPEC       finfo ;
    INT16       ch, cfn, ercode, path_shown = FALSE ;
    UCHAR       attr_str[5], cur_path[MAX_PATH_LEN], saved_DTA[48] ;
    BOOL        ynGettingKey ;

    if ( _chdir( pdir ) != 0 )
    {
        printf( "%s", pdir );
        do_quit( 2, "Directory not found" );
    }
    CurrentPath( cur_path );            // return the current directory
/*
    Searching with *.* rather than "filename_pattern" because we need to pick up
    ALL directories.  Otherwise only those directories which match the pattern
    would be searched.  The matched function is used instead to find matches.
*/
    ercode = FindFirstFile( "*.*", 23, &finfo );
    while ( ercode == 0 )
    {
        if ( finfo.fname[0] != '.' )
        {
            if ( finfo.f_at_sub_dir && (level > 0) )  // another directory
            {
                save_DTA( saved_DTA );          // save copy of DTA
                scan_dir( finfo.fname, level - 1 );
                rest_DTA( saved_DTA );          // restore the DTA
                path_shown = FALSE ;            // show path again if more files
            }
            else
            {
                //  a match, but not a sub-directory
                if ( ( matched(finfo.fname) ) && ( ! finfo.f_at_sub_dir ) )
                {
                    if ( ! path_shown )
                    {
                        printf( "\t     %s\n", cur_path );
                        path_shown = TRUE ;
                    }
                    StrFill( attr_str, '-', 5 );
                    if ( finfo.f_at_read_only )
                        attr_str[ 0 ] = 'r' ;
                    if ( finfo.f_at_hidden )
                        attr_str[ 1 ] = 'h' ;
                    if ( finfo.f_at_system )
                        attr_str[ 2 ] = 's' ;
                    if ( finfo.f_at_archive )
                        attr_str[ 3 ] = 'a' ;
                    if ( N_switch )
                    {
                        cfn = check_for_nulls( cur_path, finfo.fname );
                        if ( cfn )
                        {
                            printf( "%-12s  %-9s %-7s %10ld", finfo.fname, finfo.fdate_tx, finfo.ftime_tx, finfo.fsize );
                            if ( cfn == ERROR )
                                printf( "  ERROR opening file" );
                            if ( cfn == TRUE )
                                printf( "  NULL-padded" );
                            printf( "\n" );
                        }
                    }
                    else
                        printf( "%-12s  %-9s %-7s %10ld [%s]", finfo.fname, finfo.fdate_tx, finfo.ftime_tx, finfo.fsize, attr_str );
                    if ( verbose )
                    {
                        printf( " <E/Q/F/?>:" );
                        ynGettingKey = TRUE ;
                        while ( ynGettingKey )
                        {
                            ch = _getch();
                            switch ( toupper(ch) )
                            {
                                case 'E':           // Erase the file
                                    if ( finfo.f_at_read_only )
                                        SetAttrib( finfo.fname, -1, 0, 0, 0 ); // reset r-o attribute
                                    if ( (fl_delete( finfo.fname )) == 0 )
                                        printf( " ERASED THIS FILE\n" );
                                    else
                                        printf( " COULD NOT ERASE\a\n" );
                                    ynGettingKey = FALSE ;
                                    break ;
                                case 'Q':           // Quit, but return home
                                    printf( " Quit\n\n" );
                                    go_home();
                                    do_quit( 0, "" );
                                case 'F':           // Fast quit, stay at this dir
                                    printf( " Fast Quit" );
                                    do_quit( 0, "Remaining in directory" );
                                case '?':           // Help
                                    printf( " Help\n"
                                            "E = Erase the file\n"
                                            "Q = Quit, and return to the home directory\n"
                                            "F = Quit, but stay in the directory begin processed\n"
                                            "Any other key skips this file\n:" );
                                    break ;
                                default:
                                    printf( " Nothing done\n" );
                                    ynGettingKey = FALSE ;
                                    break ;
                            }
                        }
                    }
                    else
                    {
                        if ( ! N_switch )
                            printf( "\n" );
                    }
                }
            }
        }
        ercode = FindNextFile( &finfo );
    }
    _chdir( ".." );                     // go back up one directory
}


void    scandir_parms( STRING fl_pat, STRING dir, INT16 level )
{
    printf( "File(s)  : %s\nDirectory: %s\nLevels   : ", fl_pat, dir );
    if ( level == 32767 )
        printf( "all" );
    else
        printf( "%d", level );
    printf( "\n\n" );
}


void    scandir_init( void )
{
    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    if ( ISLOADED_ANSI() )
        ANSI_CLS();
    else
        printf( "\n\n\n" );
    logon(1);
    home_drv = CurrentDrive() - 1 ;             // save current drive & dir.
    CurrentPath( home_dir );
    CursOn();
}


void    main( int argc, char *argv[] )
{
    INT16       level = 32767, parm_no, abort, i, j, indext ;
    UCHAR       search_directory[MAX_PATH_LEN], filename_pattern[13] ;

    scandir_init();
    st_cpy( search_directory, home_dir, MAX_PATH_LEN );
    st_cpy( filename_pattern, "*.*", 13 );
    parm_no = 1 ;                       // parameter number
    abort   = FALSE ;
    for ( i = 1 ; (i < argc) && ~abort ; i++ )
    {
        if ( (*argv[i] == SLASH) || (*argv[i] == DASH) )
        {
            _strupr( argv[i] );
            switch ( *(++argv[i]) )
            {
                case 'F' :
                    st_cpy( filename_pattern, argv[++i], 13 );
                    break ;
                case 'D' :
                    st_cpy( search_directory, argv[++i], MAX_PATH_LEN );
                    break ;
                case 'L' :
                    level = Str2Int( argv[++i] );
                    break ;
                case 'T' :
                    verbose = FALSE ;
                    break ;
                case 'N' :
                    N_switch = TRUE ;
                    verbose = FALSE ;   // no VERBOSE when N switch is on
                    break ;
                default :
                    abort = TRUE ;
            }
        }
        else
        {
            switch ( parm_no )
            {
                case 1:
                    st_cpy( filename_pattern, argv[i], 13 );
                    break ;
                case 2:
                    st_cpy( search_directory, argv[i], MAX_PATH_LEN );
                    break ;
                case 3:
                    level = Str2Int( argv[i] );
                    break ;
                default:
                    abort = TRUE ;
            }
        }
        parm_no += 1 ;
    }
    fix_str( filename_pattern );
    fix_str( search_directory );
    if ( abort )
        do_help();

    //  Now set the global variables filechk & extchk.
    //  It is the pattern that is matched as the directories are searched.

    StrFill( filechk, '?', 9 );
    StrFill( extchk , '?', 4 );
    if ( ( indext = StrFindChar( '.', filename_pattern ) ) < 0 )
        indext = strlen( filename_pattern );
    j = indext ;
    for ( i = 0 ; i < indext ; i++ )
    {
        if ( filename_pattern[i] == '*' )
        {
            j = 8 ;
            break ;
        }
        else
            filechk[i] = filename_pattern[i] ;
    }
    for ( i = j ; i < 8 ; i++ )
        filechk[i] = UCHARMAX ;         // fill with nonproducable char
    j = 0 ;
    for ( i = indext + 1 ; i < ((int) strlen(filename_pattern)) ; i++ )
    {
        if ( filename_pattern[i] == '*' )
        {
            j = 3 ;
            break ;
        }
        else
            extchk[j++] = filename_pattern[i] ;
    }
    for ( i = j ; i < 3 ; i++ )
        filechk[i] = UCHARMAX ;         // fill with nonproducable char

    scandir_parms( filename_pattern, search_directory, level );
/*
    The function scan_dir is a recursive function which scans a directory.  For
    each call to scan_dir, the level is passed; no subdirectory is scanned if
    the level is not positive.  The verbosity mode and file name are globally
    definded.
*/
    if ( search_directory[1] == ':' )   // drive included
    {
        if ( ((INT16) search_directory[0]-'A') != home_drv )
            SelectDrive( (INT16) (search_directory[0]-'A') );
    }
    CurrentPath( tome_dir );            // current direc. on target drive
    scan_dir( search_directory, level );
    go_home();
    do_quit( 0, "" );
}
